import re
import sys
import unittest
from argparse import ArgumentParser


def main():
    ap = ArgumentParser(description="Rockset unittest main")
    ap.add_argument(
        "-b", "--buffer", help="Buffer output during run", action="store_true"
    )
    ap.add_argument(
        "-c", "--catch", help="Catch Ctrl-C during run", action="store_true"
    )
    ap.add_argument(
        "-f", "--failfast", help="Stop on first failure", action="store_true"
    )
    ap.add_argument(
        "--locals", help="Show local variables in tracebacks", action="store_true"
    )
    ap.add_argument("-v", "--verbose", help="Verbose output", action="store_true")
    ap.add_argument("-s", "--start-directory", help="Start directory", default=".")
    ap.add_argument("patterns", help="Test regexes to run", nargs="*")

    args = ap.parse_args()
    patterns = [re.compile(p) for p in args.patterns]

    def filter_tests(tests):
        accepted = []
        for test in tests:
            if isinstance(test, unittest.TestSuite):
                accepted.append(test)
                continue
            test_id = test.id()
            if (not patterns) or any(p.match(test_id) for p in patterns):
                accepted.append(test)

        return unittest.TestSuite(accepted)

    if args.catch:
        unittest.installHandler()

    loader = unittest.TestLoader()
    loader.suiteClass = filter_tests
    suite = loader.discover(args.start_directory)
    runner = unittest.TextTestRunner(
        verbosity=2 if args.verbose else 1, failfast=args.failfast, buffer=args.buffer
    )
    result = runner.run(suite)
    if not result.wasSuccessful():
        sys.exit(1)


if __name__ == "__main__":
    main()



import os
import sys

from docopt import docopt
from rockset.credentials import Credentials
from .command import Command


def _format_secret(value):
    if value is None:
        return None
    i = 1
    if len(value) >= 20:
        i = 4
    return "%s**********%s" % (value[:i], value[-i:])


def _prompt_api_key(api_key=None):
    return [
        {
            "key": "api_key",
            "message": "Enter Rockset API key",
            "prompt": "  API Key",
            "default": {
                "display": _format_secret(api_key),
                "value": api_key,
            },
        }
    ]


def _prompt_api_server(api_server=None):
    return [
        {
            "key": "api_server",
            "message": "Enter Rockset API server URL",
            "prompt": "  API Server",
            "default": {
                "display": api_server,
                "value": api_server,
            },
        }
    ]


def _interactive_prompt(config):
    out = {}
    for item in config:
        key = item["key"]
        message = item["message"]
        prompt = item["prompt"]
        default = item["default"]
        if "display" in default:
            prompt += " [%s]" % default["display"]
        sys.stdout.write(message + "\n")
        sys.stdout.write(prompt + ": ")
        sys.stdout.flush()
        value = input()
        value = value.strip()
        if value == "":
            value = default.get("value", "")
        out[key] = value
    return out


class Configure(Command):
    def usage(self):
        return """
usage: rock configure [--help]
       rock configure [options]
       rock configure add <profile> [options]
       rock configure update <profile> [options]
       rock configure ls
       rock configure rm <profile>
       rock configure select <profile>

Manage Rockset credential profiles. Each profile is associated
with an API key and an API endpoint.

commands:
    add, update       create a new credentials profile or update an existing one
                      Default, when no profiles exist.
    ls                list all profiles.
                      Default, when 1 or more profiles exist.
    rm                remove profile
    select            select profile to use for subsequent rock commands

arguments:
    <profile>         name of the profile you wish to create or update

options:
  -h, --help                    show this help message and exit
  -k, --api_key=API_KEY         specify API key to use for subsequent commands
  -s, --api_server=API_SERVER   specify API server URL assigned to your account
        """

    def go(self):
        creds = Credentials()

        # determine default command when none is specified
        if (
            not self.add
            and not self.update
            and not self.ls
            and not self.rm
            and not self.select
        ):

            # this is the `rock configure [options]` case
            # each case is explicitly called out for code readability
            # if api_key or api_server is specified in command-line,
            # then do `rock configure add default`
            if self.api_key or self.api_server:
                self.add = True
                self.profile = "default"
            # if this is just `rock configure` and no profiles are found
            # then do `rock configure add default`
            elif len(creds.get_all_profiles()) == 0:
                self.add = True
                self.profile = "default"
            # else just list all profiles
            else:
                self.ls = True

        # update is same as add
        if self.add or self.update:
            new_profile = self.profile
            ip = creds.get(new_profile)
            ip["api_key"] = ip.get("api_key", None)
            ip["api_server"] = ip.get("api_server", "https://api.rs2.usw2.rockset.com")
            # dont prompt if command-line options are specified
            if self.api_key or self.api_server:
                ip["api_key"] = self.api_key or ip["api_key"]
                ip["api_server"] = self.api_server or ip["api_server"]
            else:
                ip = _interactive_prompt(
                    _prompt_api_key(ip["api_key"])
                    + _prompt_api_server(ip["api_server"])
                )
            ip["profile"] = new_profile
            profile_details = creds.set(
                profile=ip["profile"],
                api_key=ip["api_key"],
                api_server=ip["api_server"],
            )
            if len(creds.get_all_profiles()) == 1:
                creds.active_profile(ip["profile"])
            creds.save()
        elif self.select:
            select_profile = self.profile
            active_profile = creds.active_profile(select_profile)
            if active_profile != select_profile:
                self.error('Could not switch to profile "%s"!' % select_profile)
            else:
                creds.save()
        elif self.rm:
            rm_profile = self.profile
            deleted_profile = creds.delete(rm_profile)
            if deleted_profile is None:
                self.error('No profile with name "%s" exists!' % rm_profile)
            else:
                creds.save()

        # print all profiles in alphabetical order
        all_profiles = creds.get_all_profiles()
        all_profile_names = list(all_profiles.keys())
        all_profile_names.sort()

        # accumulate formatted profiles
        profiles = []
        for pn in all_profile_names:
            profiles.append(
                self._format_profile(pn, all_profiles[pn], creds.active_profile())
            )

        self.print_list(
            0, profiles, field_order=["profile", "api_server", "api_key"], header=True
        )
        if self.format == "text":
            self.lprint(0, "Credentials stored in %s" % creds.creds_file)

        return 0

    def _format_profile(self, p, pd, ap):
        # prefix profile name with '*' if current
        if p == ap:
            prefix = "* "
        else:
            prefix = "  "
        pd["profile"] = prefix + p

        # don't show full api_key
        pd["api_key"] = _format_secret(pd.get("api_key", None))
        return pd



#!/usr/bin/env python3

import sys
import traceback
import json
import unittest
from setuptools import setup
from setuptools import find_packages
import setup_tests


def get_version():
    version = {}
    with open("rockset/version.json", "r") as vf:
        version = json.load(vf)
    return version


def get_long_description():
    with open("DESCRIPTION.rst", encoding="utf-8") as fd:
        long_description = fd.read()
    long_description += "\n"
    with open("RELEASE.rst", encoding="utf-8") as fd:
        long_description += fd.read()
    return long_description


setup(
    name="rockset",
    version=get_version().get("python", "???"),
    description="Rockset Python Client and `rock` CLI",
    long_description=get_long_description(),
    author="Rockset, Inc",
    author_email="api@rockset.io",
    url="https://rockset.com",
    keywords="Rockset serverless search and analytics",
    packages=find_packages(
        include=[
            "rockset",
            "rockset.*",
        ],
        exclude=[
            "rockset.internal",
            "rockset.internal.*",
            "rockset.rock.tests",
            "rockset.tests",
            "rockset.sql.tests",
        ],
    ),
    install_requires=[
        "docopt >= 0.6.2",
        "geojson >= 2.5.0",
        "protobuf >= 3.6.0",
        "pyyaml >= 5.1.2",
        "requests >= 2.19.0",
        "requests-toolbelt >= 0.9.1",
        "sqlalchemy >= 1.3.10",
        "texttable >= 0.8.7",
    ],
    entry_points={
        "console_scripts": [
            "rock = rockset.rock.main:main",
        ],
        # New versions
        "sqlalchemy.dialects": [
            "rockset= rockset.sql:RocksetDialect",
        ],
        # Version 0.5
        "sqlalchemy.databases": [
            "rockset= rockset.sql:RocksetDialect",
        ],
    },
    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: SQL",
        "Topic :: Database",
        "Topic :: Database :: Database Engines/Servers",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Shells",
        "Topic :: Internet :: WWW/HTTP :: Indexing/Search",
    ],
    package_data={
        "rockset": [
            "swagger/apiserver.yaml",
            "version.json",
        ],
    },
    test_suite="setup_tests.my_test_suite",
)



from docopt import docopt

from .command_auth import AuthCommand
from .util.type_util import IntegrationType, TypeUtil


class List(AuthCommand):
    def usage(self, hidden=False):
        return """
Usage:
    rock list [-h]
    rock list collections [--workspace=WORKSPACE]
    rock list integrations
    rock list workspaces{}

Options:
  -h, --help                      show this help message and exit
  -w, --workspace=WORKSPACE       list all collections recursively within given workspace
        """.format(
            ""
            if not hidden
            else """
    rock list c [--workspace=WORKSPACE]
    rock list col [--workspace=WORKSPACE]
    rock list collection [--workspace=WORKSPACE]
    rock list i
    rock list int
    rock list integration
    rock list w
    rock list ws
    rock list workspace"""
        )

    def parse_args(self, args):
        parsed = dict(docopt(self.usage(hidden=True), argv=args, help=False))

        if parsed["--help"]:
            ret = self.usage()
            raise SystemExit(ret.strip())

        if any(
            [parsed.get(t, "None") for t in ["c", "col", "collection", "collections"]]
        ):
            return {
                "resource": TypeUtil.TYPE_COLLECTION,
                "workspace": parsed["--workspace"],
            }

        if any(
            [parsed.get(t, "None") for t in ["i", "int", "integration", "integrations"]]
        ):
            return {"resource": TypeUtil.TYPE_INTEGRATION}

        if any([parsed.get(t, "None") for t in ["w", "ws", "workspace", "workspaces"]]):
            return {"resource": TypeUtil.TYPE_WORKSPACE}

        return {
            "resource": TypeUtil.TYPE_COLLECTION,
            "workspace": parsed["--workspace"],
        }

    def go(self):
        if self.resource == TypeUtil.TYPE_COLLECTION:
            return self.go_collections()

        if self.resource == TypeUtil.TYPE_INTEGRATION:
            return self.go_integrations()

        if self.resource == TypeUtil.TYPE_WORKSPACE:
            return self.go_workspaces()

        return self.go_collections()

    def go_collections(self):
        fields = [
            "name",
            "workspace",
            "status",
            "description",
            "created_by",
            "created_at",
            "size",
        ]
        collections = [
            c.asdict() for c in self.client.Collection.list(workspace=self.workspace)
        ]

        for c in collections:
            c["size"] = c.get("stats").get("total_size")
        collections.sort(
            key=lambda c: "{}.{}".format(c.get("workspace"), c.get("name"))
        )
        self.print_list(0, collections, fields)
        return 0

    def go_integrations(self):
        fields = ["type", "name", "description", "created_by"]
        integrations = [i.to_dict() for i in self.client.Integration.list()]
        for i in integrations:
            i["type"] = IntegrationType.parse_from_integration(i)
        integrations.sort(key=lambda i: "{}.{}".format(i.get("type"), i.get("name")))
        self.print_list(0, integrations, fields)
        return 0

    def go_workspaces(self):
        fields = ["name", "description", "collection_count", "created_by", "created_at"]
        workspaces = [w.to_dict() for w in self.client.Workspace.list()]
        for w in workspaces:
            w["is_empty"] = str(w.get("collection_count") == 0)
        workspaces.sort(key=lambda w: "{}.{}".format(w.get("is_empty"), w.get("name")))
        self.print_list(0, workspaces, fields)
        return 0
